package com.danitheskunk.skunkstream;

import com.danitheskunk.skunkworks.BaseGame;
import com.danitheskunk.skunkworks.Data;
import com.danitheskunk.skunkworks.Vec2i;
import com.danitheskunk.skunkworks.gfx.IRenderContext;
import com.danitheskunk.skunkworks.gfx.ITexture;
import com.danitheskunk.skunkworks.gfx.font.IFont;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class Skunkstream extends BaseGame {
	Data data;
	List<IElement> elements;
	HashMap<String, IFont> fonts;
	HashMap<String, ITexture> textures;
	HashMap<String, com.danitheskunk.skunkworks.gfx.NineSlice> nineSlices;

	public Skunkstream() {
		super(new Vec2i(1920, 1080), "Skunkstream v0.0");
		data = engine.loadDataWatched("c:\\dev\\skunkstream\\test.ini");
		elements = new ArrayList<>();
		fonts = new HashMap<>();
		textures = new HashMap<>();
		nineSlices = new HashMap<>();
		fonts.put("", debugFont);
		prepare();
		data.onReload(this::prepare);
	}

	public static void main(String params[]) {
		new Skunkstream().run();
	}

	void prepare() {
		elements.clear();
		var cats = data.getCategories();
		for(var cat : cats) {
			if(!cat.equals("")) {
				var type = data.get(cat, "type");
				if(type.equals("label")) {
					elements.add(new Label(
						data.getVec2i(cat, "pos"),
						data.get(cat, "text"),
						fonts.get(data.get(cat, "font")),
						data.getColor(cat, "color")
					));
				} else if(type.equals("image")) {
					var path = data.get(cat, "path");
					if(!textures.containsKey(path)) {
						textures.put(path, window.loadTexture(path));
					}
					elements.add(new Image(
						data.getVec2i(cat, "pos"),
						textures.get(path),
						data.getVec2i(cat, "size"),
						data.getColor(cat, "color")
					));
				} else if(type.equals("9slice")) {
					var path = data.get(cat, "path");
					if(!nineSlices.containsKey(path)) {
						nineSlices.put(path, window.loadNineSlice(path));
					}
					elements.add(new NineSlice(
						data.getVec2i(cat, "pos"),
						nineSlices.get(path),
						data.getVec2i(cat, "size")
					));
				} else if(type.equals("font")) {
					if(!fonts.containsKey(cat)) {
						var path = data.get(cat, "path");
						fonts.put(cat, window.loadFontTileset(path));
					}
				}
			}
		}
	}

	@Override
	protected void render(IRenderContext rc) {
		for(var element : elements) {
			element.render(rc);
		}
	}
}
